@@ -7,7 +7,7 @@ import os
 from mcomix import tools
 
 APPNAME = 'MComix'
-VERSION = '0.98'
+VERSION = '0.99'
 
 HOME_DIR = tools.get_home_directory()
 CONFIG_DIR = tools.get_config_directory()
@@ -27,7 +27,7 @@ FILEINFO_PICKLE_PATH = os.path.join(DATA_DIR, 'file.pickle')
 # Transitional - used if json preferences are (were) absent.
 PREFERENCE_PICKLE_PATH = os.path.join(CONFIG_DIR, 'preferences.pickle')
 
-ZOOM_MODE_BEST, ZOOM_MODE_WIDTH, ZOOM_MODE_HEIGHT, ZOOM_MODE_MANUAL = range(4)
+ZOOM_MODE_BEST, ZOOM_MODE_WIDTH, ZOOM_MODE_HEIGHT, ZOOM_MODE_MANUAL, ZOOM_MODE_SIZE = range(5)
 ZIP, RAR, TAR, GZIP, BZIP2, PDF, SEVENZIP, LHA = range(8)
 NORMAL_CURSOR, GRAB_CURSOR, WAIT_CURSOR, NO_CURSOR = range(4)
 LIBRARY_DRAG_EXTERNAL_ID, LIBRARY_DRAG_BOOK_ID, LIBRARY_DRAG_COLLECTION_ID = range(3)
@@ -52,7 +52,6 @@ SIZE_HUGE, SIZE_LARGE, SIZE_NORMAL, SIZE_SMALL, SIZE_TINY = MAX_LIBRARY_COVER_SI
 
 ACCEPTED_COMMENT_EXTENSIONS = ['txt', 'nfo', 'xml']
 SUPPORTED_IMAGE_REGEX = re.compile(r'\.(jpg|jpeg|png|gif|tif|tiff|bmp|ppm|pgm|pbm)\s*$', re.I)
-SUPPORTED_ARCHIVE_REGEX = re.compile(r'\.(cbz|cbr|cbt|zip|rar|tar|gz|bz2|bzip2|7z|lza|lzh)\s*$', re.I)
 
 ZIP_FORMATS = (
         ('application/x-zip', 'application/zip', 'application/x-zip-compressed', 'application/x-cbz'),
@@ -70,17 +69,7 @@ LHA_FORMATS = (
         ('application/x-lzh', 'application/x-lha', 'application/x-lzh-compressed'),
         ('*.lha', '*.lzh'))
 
-def SZIP_AVAILABLE():
-    import archive.sevenzip
-    return archive.sevenzip.SevenZipArchive.is_available()
 
-def RAR_AVAILABLE():
-    import archive.rar
-    return archive.rar.RarArchive.is_available() or SZIP_AVAILABLE()
-
-def LHA_AVAILABLE():
-    import archive.lha
-    return archive.lha.LhaArchive.is_available() or SZIP_AVAILABLE()
 
 MISSING_IMAGE_ICON = None
 try:
