@@ -10,6 +10,7 @@ from mcomix import constants
 from mcomix import portability
 from mcomix import keybindings
 
+
 class EventHandler:
 
     def __init__(self, window):
@@ -22,6 +23,11 @@ class EventHandler:
 
         #: For scrolling "off the page".
         self._extra_scroll_events = 0
+        #: If True, increment _extra_scroll_events before switchting pages
+        self._scroll_protection = False
+        #: If True, the last smart scroll action was in direction 1 (usually
+        #: horizontal). False means that direction 2 was scrolled (vertically)
+        self._last_scroll_was_direction_1 = False
 
     def resize_event(self, widget, event):
         """Handle events from resizing and moving the main window."""
@@ -48,7 +54,7 @@ class EventHandler:
         # Navigation keys that work in addition to the accelerators in ui.py
         manager.register('previous page',
             ['KP_Page_Up', 'BackSpace', '<Mod1>Left'],
-            self._window.previous_page)
+            self._previous_page_with_protection)
         manager.register('next page',
             ['KP_Page_Down', '<Mod1>Right'],
             self._window.next_page)
@@ -57,47 +63,46 @@ class EventHandler:
         manager.register('scroll left bottom',
             ['KP_1'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'left', 'vert':'bottom'})
+            kwargs={'horiz': 'left', 'vert': 'bottom'})
         manager.register('scroll middle bottom',
             ['KP_2'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'middle', 'vert':'bottom'})
+            kwargs={'horiz': 'middle', 'vert': 'bottom'})
         manager.register('scroll right bottom',
             ['KP_3'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'right', 'vert':'bottom'})
+            kwargs={'horiz': 'right', 'vert': 'bottom'})
 
         manager.register('scroll left middle',
             ['KP_4'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'left', 'vert':'middle'})
+            kwargs={'horiz': 'left', 'vert': 'middle'})
         manager.register('scroll middle',
             ['KP_5'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'middle', 'vert':'middle'})
+            kwargs={'horiz': 'middle', 'vert': 'middle'})
         manager.register('scroll right middle',
             ['KP_6'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'right', 'vert':'middle'})
+            kwargs={'horiz': 'right', 'vert': 'middle'})
 
         manager.register('scroll left top',
             ['KP_7'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'left', 'vert':'top'})
+            kwargs={'horiz': 'left', 'vert': 'top'})
         manager.register('scroll middle top',
             ['KP_8'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'middle', 'vert':'top'})
+            kwargs={'horiz': 'middle', 'vert': 'top'})
         manager.register('scroll right top',
             ['KP_9'],
             self._window.scroll_to_fixed,
-            kwargs={'horiz':'right', 'vert':'top'})
+            kwargs={'horiz': 'right', 'vert': 'top'})
 
         # Enter/exit fullscreen.
         manager.register('exit fullscreen',
             ['Escape'],
-            self._window.actiongroup.get_action('fullscreen').set_active,
-            args=(False,))
+            self.escape_event)
         manager.register('toggle fullscreen',
             ['F11'],
             self._window.actiongroup.get_action('fullscreen').activate)
@@ -150,6 +155,9 @@ class EventHandler:
     def key_press_event(self, widget, event, *args):
         """Handle key press events on the main window."""
 
+        # This is set on demand by callback functions
+        self._scroll_protection = False
+
         # Dispatch keyboard input handling
         manager = keybindings.keybinding_manager()
         manager.execute((event.keyval, event.state))
@@ -184,6 +192,13 @@ class EventHandler:
         if event.keyval in (gtk.keysyms.Control_L, gtk.keysyms.Control_R):
             self._window.imagehandler.force_single_step = False
 
+    def escape_event(self):
+        """ Determines the behavior of the ESC key. """
+        if prefs['escape quits']:
+            self._window.close_program()
+        else:
+            self._window.actiongroup.get_action('fullscreen').set_active(False)
+
     def scroll_wheel_event(self, widget, event, *args):
         """Handle scroll wheel events on the main layout area. The scroll
         wheel flips pages in best fit mode and scrolls the scrollbars
@@ -192,40 +207,31 @@ class EventHandler:
         if 'GDK_BUTTON2_MASK' in event.state.value_names:
             return
 
+        self._scroll_protection = True
+
         if event.direction == gtk.gdk.SCROLL_UP:
-
-            if prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-                self._window.previous_page()
-
-            elif prefs['zoom mode'] == constants.ZOOM_MODE_HEIGHT:
-
-                if self._window.is_manga_mode:
-                    self._scroll_with_flipping(prefs['number of pixels to scroll per mouse wheel event'], 0)
-                else:
-                    self._scroll_with_flipping(-prefs['number of pixels to scroll per mouse wheel event'], 0)
-
+            if prefs['smart scroll']:
+                self._smart_scroll_up(prefs['number of pixels to scroll per mouse wheel event'])
             else:
                 self._scroll_with_flipping(0, -prefs['number of pixels to scroll per mouse wheel event'])
 
         elif event.direction == gtk.gdk.SCROLL_DOWN:
-
-            if prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-                self._window.next_page()
-
-            elif prefs['zoom mode'] == constants.ZOOM_MODE_HEIGHT:
-
-                if self._window.is_manga_mode:
-                    self._scroll_with_flipping(-prefs['number of pixels to scroll per mouse wheel event'], 0)
-                else:
-                    self._scroll_with_flipping(prefs['number of pixels to scroll per mouse wheel event'], 0)
+            if prefs['smart scroll']:
+                self._smart_scroll_down(prefs['number of pixels to scroll per mouse wheel event'])
             else:
                 self._scroll_with_flipping(0, prefs['number of pixels to scroll per mouse wheel event'])
 
         elif event.direction == gtk.gdk.SCROLL_RIGHT:
-            self._window.next_page()
+            if not self._window.is_manga_mode:
+                self._window.next_page()
+            else:
+                self._previous_page_with_protection()
 
         elif event.direction == gtk.gdk.SCROLL_LEFT:
-            self._window.previous_page()
+            if not self._window.is_manga_mode:
+                self._previous_page_with_protection()
+            else:
+                self._window.next_page()
 
     def mouse_press_event(self, widget, event):
         """Handle mouse click events on the main layout area."""
@@ -266,7 +272,7 @@ class EventHandler:
             self._window.actiongroup.get_action('lens').set_active(False)
 
         elif event.button == 3 and event.state & gtk.gdk.MOD1_MASK:
-            self._window.previous_page()
+            self._previous_page_with_protection()
 
     def mouse_move_event(self, widget, event):
         """Handle mouse pointer movement events."""
@@ -325,8 +331,8 @@ class EventHandler:
             return
 
         # Normalize URIs
-        uris = [ portability.normalize_uri(uri) for uri in uris ]
-        paths = [ urllib.url2pathname(uri).decode('utf-8') for uri in uris ]
+        uris = [portability.normalize_uri(uri) for uri in uris]
+        paths = [urllib.url2pathname(uri).decode('utf-8') for uri in uris]
 
         if len(paths) > 1:
             self._window.filehandler.open_file(paths)
@@ -340,9 +346,11 @@ class EventHandler:
         to.
         """
 
-        if self._window.scroll(x, y) or not prefs['flip with wheel']:
-            self._extra_scroll_events = 0
+        self._scroll_protection = True
+        self._last_scroll_was_direction_1 = False
 
+        if self._window.scroll(x, y):
+            self._extra_scroll_events = 0
             return True
 
         if y > 0 or (self._window.is_manga_mode and x < 0) or (
@@ -353,86 +361,66 @@ class EventHandler:
             forwards_scroll = False
 
         if forwards_scroll:
-            self._extra_scroll_events = max(1, self._extra_scroll_events + 1)
-
-            if self._extra_scroll_events >= prefs['number of key presses before page turn']:
-                self._extra_scroll_events = 0
-                self._window.next_page()
-
-                return False
-
-            return True
+            return not self._next_page_with_protection()
         else:
-            self._extra_scroll_events = min(-1, self._extra_scroll_events - 1)
-
-            if self._extra_scroll_events <= -prefs['number of key presses before page turn']:
-                self._extra_scroll_events = 0
-
-                self._window.previous_page()
-
-                return False
-
-            return True
+            return not self._previous_page_with_protection()
 
     def _scroll_down(self):
-        """ In best fit mode, skip right to the  next page. In other modes,
-        scroll first. """
-        if not prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-            self._scroll_with_flipping(0, prefs['number of pixels to scroll per key event'])
-        else:
-            self._window.next_page()
+        """ Scrolls down. """
+        self._scroll_with_flipping(0, prefs['number of pixels to scroll per key event'])
 
     def _scroll_up(self):
-        """ In best fit mode, skip right to the  previous page. In other modes,
-        scroll first. """
-        if not prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-            self._scroll_with_flipping(0, -prefs['number of pixels to scroll per key event'])
-        else:
-            self._window.previous_page()
+        """ Scrolls up. """
+        self._scroll_with_flipping(0, -prefs['number of pixels to scroll per key event'])
 
     def _scroll_right(self):
-        """ In best fit mode, skip right to the  next page. In other modes,
-        scroll first. """
-        if not prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-            self._scroll_with_flipping(prefs['number of pixels to scroll per key event'], 0)
-        else:
-            self._window.next_page()
+        """ Scrolls right. """
+        self._scroll_with_flipping(prefs['number of pixels to scroll per key event'], 0)
 
     def _scroll_left(self):
-        """ In best fit mode, skip right to the  previous page. In other modes,
-        scroll first. """
-        if not prefs['zoom mode'] == constants.ZOOM_MODE_BEST:
-            self._scroll_with_flipping(-prefs['number of pixels to scroll per key event'], 0)
-        else:
-            self._window.previous_page()
+        """ Scrolls left. """
+        self._scroll_with_flipping(-prefs['number of pixels to scroll per key event'], 0)
 
-    def _smart_scroll_down(self):
+    def _smart_scroll_down(self, small_step=None):
         """ Smart scrolling. """
 
-        x_step, y_step = self._window.get_visible_area_size()
-        distance = prefs['scroll step distance']
-        x_step = int(x_step * distance)
-        y_step = int(y_step * distance)
+        width, height = self._window.get_visible_area_size()
+        distance = prefs['smart scroll percentage']
+
+        if small_step is None:
+            x_step_small = x_step_large = int(width * distance)
+            y_step_small = y_step_large = int(height * distance)
+        else:
+            x_step_small = small_step
+            y_step_small = small_step
+            x_step_large = int(width * distance)
+            y_step_large = int(height * distance)
+
+            if prefs['invert smart scroll']:
+                x_step_small, y_step_small = y_step_small, x_step_small
+                x_step_large, y_step_large = y_step_large, x_step_small
 
         if self._window.is_manga_mode:
-            x_step *= -1
+            x_step_small *= -1
+            x_step_large *= -1
 
-        if not prefs["smart space scroll"]:
-            if (prefs['zoom mode'] == constants.ZOOM_MODE_BEST or
-                not self._window.scroll(0, y_step)):
-                self._window.next_page()
+        if not prefs["smart scroll"]:
+            if not self._window.scroll(0, y_step_small):
+                self._next_page_with_protection()
             return
 
         if self._window.displayed_double():
             if self._window.is_on_first_page():
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(x_step_small, 0, 'first')
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
 
-                if not self._window.scroll(x_step, 0, 'first'):
-
-                    if not self._window.scroll(0, y_step):
+                    if not self._window.scroll(0, scroll_size):
 
                         if not self._window.scroll_to_fixed(
                           horiz='startsecond'):
-                            self._window.next_page()
+                            self._next_page_with_protection()
                         else:
                             self._window.scroll_to_fixed(
                                     vert='top')
@@ -442,11 +430,13 @@ class EventHandler:
                         self._window.scroll_to_fixed(
                             horiz='startfirst')
             else:
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(x_step_small, 0, 'second')
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
 
-                if not self._window.scroll(x_step, 0, 'second'):
-
-                    if not self._window.scroll(0, y_step):
-                        self._window.next_page()
+                    if not self._window.scroll(0, scroll_size):
+                        self._next_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(
                             horiz='startsecond')
@@ -454,59 +444,79 @@ class EventHandler:
             # When a double page is displayed, scroll left/right,
             # then top/bottom
             if not prefs['invert smart scroll']:
-                if not self._window.scroll(x_step, 0):
-
-                    if not self._window.scroll(0, y_step):
-                        self._window.next_page()
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(x_step_small, 0)
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
+                    if not self._window.scroll(0, scroll_size):
+                        self._next_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(
                             horiz='startfirst')
             # Scroll top/bottom, then left/right
             else:
-                if not self._window.scroll(0, y_step):
-                    if not self._window.scroll(x_step, 0):
-                        self._window.next_page()
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(0, y_step_small)
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and x_step_large or x_step_small
+                    if not self._window.scroll(scroll_size, 0):
+                        self._next_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(
                             horiz='startsecond', vert='top')
 
-
-    def _smart_scroll_up(self):
+    def _smart_scroll_up(self, small_step=None):
         """ Reversed smart scrolling. """
 
-        x_step, y_step = self._window.get_visible_area_size()
-        distance = prefs['scroll step distance']
-        x_step = int(x_step * distance)
-        y_step = int(y_step * distance)
+        width, height = self._window.get_visible_area_size()
+        distance = prefs['smart scroll percentage']
+
+        if small_step is None:
+            x_step_small = x_step_large = int(width * distance)
+            y_step_small = y_step_large = int(height * distance)
+        else:
+            x_step_small = small_step
+            y_step_small = small_step
+            x_step_large = int(width * distance)
+            y_step_large = int(height * distance)
+
+            if prefs['invert smart scroll']:
+                x_step_small, y_step_small = y_step_small, x_step_small
+                x_step_large, y_step_large = y_step_large, x_step_small
 
         if self._window.is_manga_mode:
-            x_step *= -1
+            x_step_small *= -1
+            x_step_large *= -1
 
-        if not prefs["smart space scroll"]:
-            if (prefs['zoom mode'] == constants.ZOOM_MODE_BEST or
-                not self._window.scroll(0, -y_step)):
-                self._window.previous_page()
+        if not prefs["smart scroll"]:
+            if not self._window.scroll(0, -y_step_small):
+                self._previous_page_with_protection()
             return
 
         if self._window.displayed_double():
             if self._window.is_on_first_page():
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(-x_step_small, 0, 'first')
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
 
-                if not self._window.scroll(-x_step, 0, 'first'):
-
-                    if not self._window.scroll(0, -y_step):
-                        self._window.previous_page()
+                    if not self._window.scroll(0, -scroll_size):
+                        self._previous_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(
                             horiz='endfirst')
 
             else:
-                if not self._window.scroll(-x_step, 0, 'second'):
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(-x_step_small, 0, 'second')
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
 
-                    if not self._window.scroll(0, -y_step):
+                    if not self._window.scroll(0, -scroll_size):
 
                         if not self._window.scroll_to_fixed(
                           horiz='endfirst'):
-                            self._window.previous_page()
+                            self._previous_page_with_protection()
 
                         else:
                             self._window.scroll_to_fixed(
@@ -519,21 +529,79 @@ class EventHandler:
             # When a double page is displayed, scroll left/right,
             # then top/bottom
             if not prefs['invert smart scroll']:
-                if not self._window.scroll(-x_step, 0):
-
-                    if not self._window.scroll(0, -y_step):
-                        self._window.previous_page()
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(-x_step_small, 0)
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and y_step_large or y_step_small
+                    if not self._window.scroll(0, -scroll_size):
+                        self._previous_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(horiz='endfirst')
             # Scroll top/bottom, then left/right
             else:
-                if not self._window.scroll(0, -y_step):
-                    if not self._window.scroll(-x_step, 0):
-                        self._window.previous_page()
+                last_scroll = self._last_scroll_was_direction_1
+                self._last_scroll_was_direction_1 = self._window.scroll(0, -y_step_small)
+                if not self._last_scroll_was_direction_1:
+                    scroll_size = last_scroll and x_step_large or x_step_small
+                    if not self._window.scroll(-scroll_size, 0):
+                        self._previous_page_with_protection()
                     else:
                         self._window.scroll_to_fixed(
                             horiz='startfirst', vert='bottom')
 
+    def _next_page_with_protection(self):
+        """ Advances to the next page. If L{_scroll_protection} is enabled,
+        this method will only advance if enough scrolling attempts have been made.
+
+        @return: True when the page was flipped."""
+
+        if not prefs['flip with wheel']:
+            self._extra_scroll_events = 0
+            return False
+
+        if (not self._scroll_protection
+            or self._extra_scroll_events >= prefs['number of key presses before page turn'] - 1
+            or not self._window.is_scrollable_vertically()):
+
+            self._extra_scroll_events = 0
+            self._window.next_page()
+            return True
+
+        elif (self._scroll_protection):
+            self._extra_scroll_events = max(1, self._extra_scroll_events + 1)
+            return False
+
+        else:
+            # This path should not be reached.
+            assert False, "Programmer is moron, incorrect assertion."
+
+    def _previous_page_with_protection(self):
+        """ Goes back to the previous page. If L{_scroll_protection} is enabled,
+        this method will only go back if enough scrolling attempts have been made.
+
+        @return: True when the page was flipped."""
+
+        if not prefs['flip with wheel']:
+            self._extra_scroll_events = 0
+            return False
+
+        if (not self._scroll_protection
+            or self._extra_scroll_events <= -prefs['number of key presses before page turn'] + 1
+            or not self._window.is_scrollable_vertically()):
+
+            self._extra_scroll_events = 0
+            self._window.previous_page()
+            return True
+
+        elif (self._scroll_protection):
+            self._extra_scroll_events = min(-1, self._extra_scroll_events - 1)
+            return False
+
+        else:
+            # This path should not be reached.
+            assert False, "Programmer is moron, incorrect assertion."
+
+
 def _get_latest_event_of_same_type(event):
     """Return the latest event in the event queue that is of the same type
     as <event>, or <event> itself if no such events are in the queue. All
