@@ -8,10 +8,12 @@ import pango
 
 from mcomix.preferences import prefs
 from mcomix import image_tools
+from mcomix import archive_tools
 from mcomix import labels
 from mcomix import constants
 from mcomix import log
 from mcomix import thumbnail_tools
+from mcomix import message_dialog
 
 mimetypes.init()
 
@@ -85,13 +87,13 @@ class _BaseFileChooserDialog(gtk.Dialog):
         # extractor availability.
         mimetypes = constants.ZIP_FORMATS[0] + constants.TAR_FORMATS[0]
         patterns = constants.ZIP_FORMATS[1] + constants.TAR_FORMATS[1]
-        if constants.RAR_AVAILABLE():
+        if archive_tools.rar_available():
             mimetypes += constants.RAR_FORMATS[0]
             patterns += constants.RAR_FORMATS[1]
-        if constants.SZIP_AVAILABLE():
+        if archive_tools.szip_available():
             mimetypes += constants.SZIP_FORMATS[0]
             patterns += constants.SZIP_FORMATS[1]
-        if constants.LHA_AVAILABLE():
+        if archive_tools.lha_available():
             mimetypes += constants.LHA_FORMATS[0]
             patterns += constants.LHA_FORMATS[1]
 
@@ -104,15 +106,15 @@ class _BaseFileChooserDialog(gtk.Dialog):
         self.add_filter(_('Tar archives'),
             *constants.TAR_FORMATS)
 
-        if constants.RAR_AVAILABLE():
+        if archive_tools.rar_available():
             self.add_filter(_('RAR archives'),
                 *constants.RAR_FORMATS)
 
-        if constants.SZIP_AVAILABLE():
+        if archive_tools.szip_available():
             self.add_filter(_('7z archives'),
                 *constants.SZIP_FORMATS)
 
-        if constants.LHA_AVAILABLE():
+        if archive_tools.lha_available():
             self.add_filter(_('LHA archives'),
                 *constants.LHA_FORMATS)
 
@@ -221,15 +223,13 @@ class _BaseFileChooserDialog(gtk.Dialog):
                 not os.path.isdir(first_path) and
                 os.path.exists(first_path)):
 
-                overwrite_dialog = gtk.MessageDialog(None, 0,
-                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL,
+                overwrite_dialog = message_dialog.MessageDialog(None, 0,
+                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL)
+                overwrite_dialog.set_text(
                     _("A file named '%s' already exists. Do you want to replace it?") %
-                    os.path.basename(first_path))
-
-                overwrite_dialog.format_secondary_text(
+                        os.path.basename(first_path),
                     _('Replacing it will overwrite its contents.'))
                 response = overwrite_dialog.run()
-                overwrite_dialog.destroy()
 
                 if response != gtk.RESPONSE_OK:
                     self.emit_stop_by_name('response')
