@@ -51,10 +51,10 @@ def fit_in_rectangle(src, width, height, scale_up=False, rotation=0):
         if src.get_has_alpha():
             if prefs['checkered bg for transparent images']:
                 src = src.composite_color_simple(src_width, src_height,
-                    gtk.gdk.INTERP_TILES, 255, 8, 0x777777, 0x999999)
+                    prefs['scaling quality'], 255, 8, 0x777777, 0x999999)
             else:
                 src = src.composite_color_simple(src_width, src_height,
-                    gtk.gdk.INTERP_TILES, 255, 1024, 0xFFFFFF, 0xFFFFFF)
+                    prefs['scaling quality'], 255, 1024, 0xFFFFFF, 0xFFFFFF)
     else:
         if float(src_width) / width > float(src_height) / height:
             height = int(max(src_height * width / src_width, 1))
@@ -64,12 +64,12 @@ def fit_in_rectangle(src, width, height, scale_up=False, rotation=0):
         if src.get_has_alpha():
             if prefs['checkered bg for transparent images']:
                 src = src.composite_color_simple(width, height,
-                    gtk.gdk.INTERP_TILES, 255, 8, 0x777777, 0x999999)
+                    prefs['scaling quality'], 255, 8, 0x777777, 0x999999)
             else:
                 src = src.composite_color_simple(width, height,
-                    gtk.gdk.INTERP_TILES, 255, 1024, 0xFFFFFF, 0xFFFFFF)
+                    prefs['scaling quality'], 255, 1024, 0xFFFFFF, 0xFFFFFF)
         else:
-            src = src.scale_simple(width, height, gtk.gdk.INTERP_TILES)
+            src = src.scale_simple(width, height, prefs['scaling quality'])
 
     if rotation == 90:
         src = src.rotate_simple(gtk.gdk.PIXBUF_ROTATE_CLOCKWISE)
@@ -385,11 +385,8 @@ def combine_pixbufs( pixbuf1, pixbuf2, are_in_manga_mode ):
 def is_image_file(path):
     """Return True if the file at <path> is an image file recognized by PyGTK.
     """
-    if os.path.isfile(path):
-        ext = os.path.splitext(path)[1][1:].lower()
-        ext_index = bisect.bisect_left(_supported_formats, ext)
-        return ext_index != len(_supported_formats) and _supported_formats[ext_index] == ext
-    else:
-        return False
+    ext = os.path.splitext(path)[1][1:].lower()
+    ext_index = bisect.bisect_left(_supported_formats, ext)
+    return ext_index != len(_supported_formats) and _supported_formats[ext_index] == ext
 
 # vim: expandtab:sw=4:ts=4
