@@ -28,7 +28,7 @@ class _LibraryBackend:
 
     #: Current version of the library database structure.
     # See method _upgrade_database() for changes between versions.
-    DB_VERSION = 3
+    DB_VERSION = 4
 
     def __init__(self):
 
@@ -41,7 +41,7 @@ class _LibraryBackend:
             return row
 
         self._con = dbapi2.connect(constants.LIBRARY_DATABASE_PATH,
-            check_same_thread=False)
+            check_same_thread=False, isolation_level=None)
         self._con.row_factory = row_factory
 
         self.watchlist = backend_types._WatchList(self)
@@ -278,7 +278,7 @@ class _LibraryBackend:
 
                 book = backend_types._Book(book_id, name, path, pages,
                         format, size, datetime.datetime.now().isoformat())
-                self.book_added(book)
+                self.book_added(book, collection)
 
             cursor.close()
 
@@ -291,7 +291,7 @@ class _LibraryBackend:
             return False
 
     @callback.Callback
-    def book_added(self, book):
+    def book_added(self, book, collection):
         """ Event that triggers when a new book is successfully added to the
         library.
         @param book: L{_Book} instance of the newly added book.
@@ -496,6 +496,14 @@ class _LibraryBackend:
                     from book_old''')
                 self._con.execute('''drop table book_old''')
 
+            if 3 in upgrades:
+                # Added field 'recursive' to table 'watchlist'
+                self._con.execute('''alter table watchlist rename to watchlist_old''')
+                self._create_table_watchlist()
+                self._con.execute('''insert into watchlist
+                    (path, collection, recursive)
+                    select path, collection, 0 from watchlist_old''')
+                self._con.execute('''drop table watchlist_old''')
 
             self._con.execute('''update info set value = ? where key = 'version' ''',
                               (str(_LibraryBackend.DB_VERSION),))
@@ -533,7 +541,8 @@ class _LibraryBackend:
     def _create_table_watchlist(self):
         self._con.execute('''create table if not exists watchlist (
             path text primary key,
-            collection integer references collection (id) on delete set null)''')
+            collection integer references collection (id) on delete set null,
+            recursive boolean not null)''')
 
 
 _backend = None
