@@ -16,6 +16,7 @@ from mcomix import portability
 from mcomix import i18n
 from mcomix import status
 from mcomix import log
+from mcomix import message_dialog
 from mcomix.library.pixbuf_cache import get_pixbuf_cache
 
 _dialog = None
@@ -240,9 +241,14 @@ class _BookArea(gtk.ScrolledWindow):
 
         self._iconview.draw_thumbnails_on_screen()
 
-    def _new_book_added(self, book):
+    def _new_book_added(self, book, collection):
         """ Callback function for L{LibraryBackend.book_added}. """
-        self.add_books([book])
+        if collection is None:
+            collection = _COLLECTION_ALL
+
+        if (collection == self._library.collection_area.get_current_collection() or
+            self._library.collection_area.get_current_collection() == _COLLECTION_ALL):
+            self.add_books([book])
 
     def remove_book_at_path(self, path):
         """Remove the book at <path> from the ListStore (and thus from
@@ -353,11 +359,10 @@ class _BookArea(gtk.ScrolledWindow):
         elif name == 'tiny':
             prefs['library cover size'] = constants.SIZE_TINY
         elif name == 'custom':
-            dialog = gtk.MessageDialog(self._library, gtk.DIALOG_DESTROY_WITH_PARENT,
-                    gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
-            dialog.set_markup('<span weight="bold" size="larger">' +
-                _('Set library cover size') +
-                '</span>')
+            dialog = message_dialog.MessageDialog(self._library, gtk.DIALOG_DESTROY_WITH_PARENT,
+                gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_OK)
+            dialog.set_auto_destroy(False)
+            dialog.set_text(_('Set library cover size'))
 
             # Add adjustment scale
             adjustment = gtk.Adjustment(prefs['library cover size'], 20,
@@ -373,7 +378,6 @@ class _BookArea(gtk.ScrolledWindow):
                 cover_size_scale.add_mark(mark, gtk.POS_TOP, None)
 
             dialog.get_message_area().pack_end(cover_size_scale)
-            dialog.show_all()
             response = dialog.run()
             size = int(adjustment.get_value())
             dialog.destroy()
@@ -473,19 +477,17 @@ class _BookArea(gtk.ScrolledWindow):
 
         if request_response:
 
-            choice_dialog = gtk.MessageDialog(self._library, 0,
+            choice_dialog = message_dialog.MessageDialog(self._library, 0,
                 gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO)
-            choice_dialog.set_markup('<span weight="bold" size="larger">' +
-                _('Remove books from the library?') +
-                '</span>'
-            )
-            choice_dialog.format_secondary_text(
+            choice_dialog.set_default_response(gtk.RESPONSE_YES)
+            choice_dialog.set_should_remember_choice('library-remove-book-from-disk',
+                (gtk.RESPONSE_YES,))
+            choice_dialog.set_text(
+                _('Remove books from the library?'),
                 _('The selected books will be removed from the library and '
                   'permanently deleted. Are you sure that you want to continue?')
             )
-            choice_dialog.set_default_response(gtk.RESPONSE_YES)
             response = choice_dialog.run()
-            choice_dialog.destroy()
 
         # if no request is needed or the user has told us they definitely want to delete the book
         if not request_response or (request_response and response == gtk.RESPONSE_YES):
@@ -591,7 +593,7 @@ class _BookArea(gtk.ScrolledWindow):
             cover = constants.MISSING_IMAGE_ICON
 
         cover = cover.scale_simple(max(0, cover.get_width() // 2),
-            max(0, cover.get_height() // 2), gtk.gdk.INTERP_TILES)
+            max(0, cover.get_height() // 2), prefs['scaling quality'])
         cover = image_tools.add_border(cover, 1, 0xFFFFFFFF)
         cover = image_tools.add_border(cover, 1)
 
@@ -602,7 +604,7 @@ class _BookArea(gtk.ScrolledWindow):
                 max(30, cover_width + 15), max(30, cover_height + 10))
             pointer.fill(0x00000000)
             cover.composite(pointer, 0, 0, cover_width, cover_height, 0, 0,
-            1, 1, gtk.gdk.INTERP_TILES, 255)
+            1, 1, prefs['scaling quality'], 255)
             im = Image.new('RGBA', (30, 30), 0x00000000)
             draw = ImageDraw.Draw(im)
             draw.polygon(
@@ -617,7 +619,7 @@ class _BookArea(gtk.ScrolledWindow):
             circle = image_tools.pil_to_pixbuf(im)
             circle.composite(pointer, max(0, cover_width - 15),
                 max(0, cover_height - 20), 30, 30, max(0, cover_width - 15),
-                max(0, cover_height - 20), 1, 1, gtk.gdk.INTERP_TILES, 255)
+                max(0, cover_height - 20), 1, 1, prefs['scaling quality'], 255)
         else:
             pointer = cover
 
