@@ -34,6 +34,7 @@ class _LibraryDialog(gtk.Window):
         self.resize(prefs['lib window width'], prefs['lib window height'])
         self.set_title(_('Library'))
         self.connect('delete_event', self.close)
+        self.connect('key-press-event', self._key_press_event)
 
         self.filter_string = None
         self._file_handler = file_handler
@@ -61,15 +62,13 @@ class _LibraryDialog(gtk.Window):
         self.show_all()
         self.present()
 
-        self.scan_for_new_files()
-
     def open_book(self, books, keep_library_open=False):
         """Open the book with ID <book>."""
 
         paths = [ self.backend.get_book_path(book) for book in books ]
 
         if not keep_library_open:
-            self.close()
+            self.hide()
 
         self._window.present()
 
@@ -95,10 +94,14 @@ class _LibraryDialog(gtk.Window):
                 collection_name = None
 
             self.add_books(filelist, collection_name)
-            message = i18n.get_translation().ungettext(
-                "Added new book '%(bookname)s' from directory '%(directory)s'.",
-                "Added %(count)d new books from directory '%(directory)s'.",
-                len(filelist))
+
+            if len(filelist) == 1:
+                message = _("Added new book '%(bookname)s' "
+                    "from directory '%(directory)s'.")
+            else:
+                message = _("Added %(count)d new books "
+                    "from directory '%(directory)s'.")
+
             self.set_status_message(message % {'directory': watchentry.directory,
                 'count': len(filelist), 'bookname': os.path.basename(filelist[0])})
         else:
@@ -149,6 +152,12 @@ class _LibraryDialog(gtk.Window):
         if collection_id is not None:
             prefs['last library collection'] = collection_id
 
+    def _key_press_event(self, widget, event, *args):
+        """ Handle key press events for closing the library on Escape press. """
+
+        if event.keyval == gtk.keysyms.Escape:
+            self.hide()
+
 
 def open_dialog(action, window):
     global _dialog
@@ -164,6 +173,10 @@ def open_dialog(action, window):
     else:
         _dialog.present()
 
+    if prefs['scan for new books on library startup']:
+        _dialog.scan_for_new_files()
+
+
 def _close_dialog(*args):
     global _dialog
 
