@@ -44,9 +44,9 @@ prefs = {
     'slideshow delay': 3000,
     'slideshow can go to next archive': True,
     'number of pixels to scroll per slideshow event': 50,
-    'smart space scroll': True,
+    'smart scroll': True,
     'invert smart scroll': False,
-    'scroll step distance': 0.9,
+    'smart scroll percentage': 0.5,
     'flip with wheel': True,
     'store recent file info': constants.STORE_LAST_PATH,
     'hide all': False,
@@ -66,7 +66,7 @@ prefs = {
     'vertical flip': False,
     'horizontal flip': False,
     'keep transformation': False,
-    'replace bookmark response': None,
+    'stored dialog choices': {},
     'brightness': 1.0,
     'contrast': 1.0,
     'saturation': 1.0,
@@ -75,6 +75,8 @@ prefs = {
     'max pages to cache': 7,
     'window height': 600,
     'window width': 500,
+    'pageselector height': -1,
+    'pageselector width': -1,
     'library cover size': 125,
     'auto add books into collections': True,
     'last library collection': None,
@@ -86,7 +88,12 @@ prefs = {
     'statusbar fields': constants.STATUS_PAGE | constants.STATUS_RESOLUTION | \
                         constants.STATUS_PATH | constants.STATUS_FILENAME,
     'max threads': 3,
-    'wrap mouse scroll': False
+    'wrap mouse scroll': False,
+    'scaling quality': 1,  # gtk.gdk.INTERP_TILES
+    'escape quits': False,
+    'fit to size mode': constants.ZOOM_MODE_HEIGHT,
+    'fit to size px': 1800,
+    'scan for new books on library startup': True
 }
 
 def read_preferences_file():
@@ -99,13 +106,20 @@ def read_preferences_file():
             config_file = open(constants.PREFERENCE_PATH, 'r')
             saved_prefs = json.load(config_file)
             config_file.close()
-        except Exception:
+        except:
             # Gettext might not be installed yet at this point.
             corrupt_name = "%s.broken" % constants.PREFERENCE_PATH
             print ('! Corrupt preferences file, moving to "%s".' %
                    corrupt_name)
             if os.path.isfile(corrupt_name):
                 os.unlink(corrupt_name)
+
+            try:
+                # File cannot be moved without closing it first
+                config_file.close()
+            except:
+                pass
+
             os.rename(constants.PREFERENCE_PATH, corrupt_name)
 
     elif os.path.isfile(constants.PREFERENCE_PICKLE_PATH):
