@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 """preferences_dialog.py - Preferences dialog."""
 
 import operator
@@ -8,6 +10,7 @@ from mcomix.preferences import prefs
 from mcomix import preferences_page
 from mcomix import image_tools
 from mcomix import constants
+from mcomix import message_dialog
 
 _dialog = None
 
@@ -20,8 +23,9 @@ class _PreferencesDialog(gtk.Dialog):
     def __init__(self, window):
         gtk.Dialog.__init__(self, _('Preferences'), window, gtk.DIALOG_MODAL)
 
-        reset = self.add_button(_('Clear dialog choices'), constants.RESPONSE_REVERT_TO_DEFAULT)
+        self.reset_button = reset = self.add_button(_('Clear dialog choices'), constants.RESPONSE_REVERT_TO_DEFAULT)
         reset.set_tooltip_text(_('Clears all dialog choices that you have previously chosen not to be asked again.'))
+        reset.set_sensitive(len(prefs['stored dialog choices']) > 0)
         self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
 
         self._window = window
@@ -139,12 +143,14 @@ class _PreferencesDialog(gtk.Dialog):
         page = preferences_page._PreferencePage(None)
         page.new_section(_('Scroll'))
         smart_space_button = gtk.CheckButton(
-            _('Use smart space key scrolling'))
-        smart_space_button.set_active(prefs['smart space scroll'])
+            _('Use smart scrolling'))
+        smart_space_button.set_active(prefs['smart scroll'])
         smart_space_button.connect('toggled', self._check_button_cb,
-            'smart space scroll')
+            'smart scroll')
         smart_space_button.set_tooltip_text(
-            _('Use smart scrolling with the space key. Normally the space key scrolls only right down (or up when shift is pressed), but with this preference set it also scrolls sideways and so tries to follow the natural reading order of the comic book.'))
+            _('With this preference set, the space key and mouse wheel '
+              'do not only scroll down or up, but also sideways and so '
+              'try to follow the natural reading order of the comic book.'))
         page.add_row(smart_space_button)
 
         flip_with_wheel_button = gtk.CheckButton(
@@ -192,6 +198,17 @@ class _PreferencesDialog(gtk.Dialog):
             _('Set the number of pixels to scroll on a page when using a mouse wheel.'))
         page.add_row(label, scroll_key_spinner)
 
+        smart_scroll_label = gtk.Label(_('Fraction of page to scroll '
+            'per space key press (in percent):'))
+        adjustment = gtk.Adjustment(int(prefs['smart scroll percentage'] * 100),
+            1, 100, 1, 5)
+        spinner = gtk.SpinButton(adjustment, digits=0)
+        spinner.connect('value-changed', self._spinner_cb,
+            'smart scroll percentage')
+        spinner.set_tooltip_text(_('Sets the percentage by which the page '
+            'will be scrolled down or up when the space key is pressed.'))
+        page.add_row(smart_scroll_label, spinner)
+
         label = gtk.Label(_('Number of "steps" to take before flipping the page:'))
         adjustment = gtk.Adjustment(prefs['number of key presses before page turn'], 1, 100, 1, 3)
         flipping_spinner = gtk.SpinButton(adjustment, digits=0)
@@ -258,6 +275,20 @@ class _PreferencesDialog(gtk.Dialog):
             'hide all in fullscreen')
         page.add_row(hide_in_fullscreen_button)
 
+        page.new_section(_('Fit to size mode'))
+
+        fitside_label = gtk.Label(_('Fit to width or height:'))
+        fitmode = self._create_fitmode_control()
+        page.add_row(fitside_label, fitmode)
+
+        fitsize_label = gtk.Label(_('Fixed size for this mode:'))
+        adjustment = gtk.Adjustment(prefs['fit to size px'], 10, 10000, 10, 50)
+        fitsize_spinner = gtk.SpinButton(adjustment, digits=0)
+        fitsize_spinner.set_size_request(80, -1)
+        fitsize_spinner.connect('value-changed', self._spinner_cb,
+            'fit to size px')
+        page.add_row(fitsize_label, fitsize_spinner)
+
         page.new_section(_('Slideshow'))
         label = gtk.Label(_('Slideshow delay (in seconds):'))
         adjustment = gtk.Adjustment(prefs['slideshow delay'] / 1000.0,
@@ -298,6 +329,13 @@ class _PreferencesDialog(gtk.Dialog):
             _('Automatically rotate images when an orientation is specified in the image metadata, such as in an Exif tag.'))
         page.add_row(auto_rotate_button)
 
+        page.new_section(_('Image quality'))
+        label = gtk.Label(_('Scaling mode'))
+        scaling_box = self._create_scaling_quality_combobox()
+        scaling_box.set_tooltip_text(
+            _('Changes how images are scaled. Slower algorithms result in higher quality resizing, but longer page loading times.'))
+        page.add_row(label, scaling_box)
+
         return page
 
     def _init_advanced_tab(self):
@@ -306,11 +344,20 @@ class _PreferencesDialog(gtk.Dialog):
         # ----------------------------------------------------------------
         page = preferences_page._PreferencePage(None)
 
-        page.new_section(_('User interface language'))
+        page.new_section(_('User interface'))
         label = gtk.Label(_('Language (needs restart):'))
         language_box = self._create_language_control()
         page.add_row(label, language_box)
 
+        esc_quits = gtk.CheckButton(_('Escape key closes program'))
+        esc_quits.set_active(prefs['escape quits'])
+        esc_quits.connect('toggled', self._check_button_cb,
+            'escape quits')
+        esc_quits.set_tooltip_text(
+            _('When active, the ESC key closes the program, instead of only '
+              'disabling fullscreen mode.'))
+        page.add_row(esc_quits)
+
         page.new_section(_('File order'))
 
         label = gtk.Label(_('Order files by:'))
@@ -375,7 +422,8 @@ class _PreferencesDialog(gtk.Dialog):
 
         elif response == constants.RESPONSE_REVERT_TO_DEFAULT:
             # Reset stored choices
-            prefs['replace bookmark response'] = None
+            prefs['stored dialog choices'] = {}
+            self.reset_button.set_sensitive(False)
 
         else:
             # Other responses close the dialog, e.g. clicking the X icon on the dialog.
@@ -383,31 +431,33 @@ class _PreferencesDialog(gtk.Dialog):
 
     def _create_language_control(self):
         """ Creates and returns the combobox for language selection. """
+        # Source: http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
         languages = [
             (_('Auto-detect (Default)'), 'auto'),
-            (_('Catalan'), 'ca'),
-            (_('Czech'), 'cs'),
-            (_('German'), 'de'),
-            (_('Greek'), 'el'),
-            (_('English'), 'en'),
-            (_('Spanish'), 'es'),
-            (_('Persian'), 'fa'),
-            (_('French'), 'fr'),
-            (_('Galician'), 'gl'),
-            (_('Croatian'), 'hr'),
-            (_('Hungarian'), 'hu'),
-            (_('Indonesian'), 'id'),
-            (_('Italian'), 'it'),
-            (_('Japanese'), 'ja'),
-            (_('Korean'), 'ko'),
-            (_('Dutch'), 'nl'),
-            (_('Polish'), 'pl'),
-            (_('Portuguese'), 'pt_BR'),
-            (_('Russian'), 'ru'),
-            (_('Swedish'), 'sv'),
-            (_('Ukrainian'), 'uk'),
-            (_('Chinese (simplified)'), 'zh_CN'),
-            (_('Chinese (traditional)'), 'zh_TW')]
+            ('Català', 'ca'),  # Catalan
+            ('čeština', 'cs'),  # Czech
+            ('Deutsch', 'de'),  # German
+            ('ελληνικά', 'el'),  # Greek
+            ('English', 'en'),  # English
+            ('Español', 'es'),  # Spanish
+            ('فارسی', 'fa'),  # Persian
+            ('Français', 'fr'), # French
+            ('Galego', 'gl'),  # Galician
+            ('עברית', 'he'),  # Hebrew
+            ('Hrvatski jezik', 'hr'),  # Croatian
+            ('Magyar', 'hu'),  # Hungarian
+            ('Bahasa Indonesia', 'id'),  # Indonesian
+            ('Italiano', 'it'),  # Italian
+            ('日本語', 'ja'),  # Japanese
+            ('한국어', 'ko'),  # Korean
+            ('Nederlands', 'nl'),  # Dutch
+            ('Język polski', 'pl'),  # Polish
+            ('Português', 'pt_BR'),  # Portuguese
+            ('pусский язык', 'ru'),  # Russian
+            ('Svenska', 'sv'),  # Swedish
+            ('українська мова', 'uk'),  # Ukrainian
+            ('簡體中文', 'zh_CN'),  # Chinese (simplified)
+            ('正體中文', 'zh_TW')]  # Chinese (traditional)
         languages.sort(key=operator.itemgetter(0))
 
         box = self._create_combobox(languages, prefs['language'],
@@ -445,6 +495,28 @@ class _PreferencesDialog(gtk.Dialog):
             prefs['virtual double page for fitting images'] = value
             self._window.draw_image()
 
+    def _create_fitmode_control(self):
+        """ Combobox for fit to size mode """
+        items = (
+                (_('Fit to width'), constants.ZOOM_MODE_WIDTH),
+                (_('Fit to height'), constants.ZOOM_MODE_HEIGHT))
+
+        box = self._create_combobox(items,
+                prefs['fit to size mode'],
+                self._fit_to_size_changed_cb)
+
+        return box
+
+    def _fit_to_size_changed_cb(self, combobox, *args):
+        """ Change to 'Fit to size' pixels """
+        iter = combobox.get_active_iter()
+        if combobox.get_model().iter_is_valid(iter):
+            value = combobox.get_model().get_value(iter, 1)
+
+            if prefs['fit to size mode'] != value:
+                prefs['fit to size mode'] = value
+                self._window.change_zoom_mode()
+
     def _create_sort_by_control(self):
         """ Creates the ComboBox control for selecting archive sort by options. """
         sortkey_items = (
@@ -528,22 +600,41 @@ class _PreferencesDialog(gtk.Dialog):
             and (self._window.uimanager.recent.count() > 0
                  or self._window.filehandler.last_read_page.count() > 0)):
 
-            dialog = gtk.MessageDialog(self, gtk.DIALOG_MODAL,
+            dialog = message_dialog.MessageDialog(self, gtk.DIALOG_MODAL,
                 gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO)
             dialog.set_default_response(gtk.RESPONSE_YES)
-            dialog.set_markup('<span weight="bold" size="larger">' +
-                _('Delete information about recently opened files?') +
-                '</span>')
-            dialog.format_secondary_text(
+            dialog.set_text(
+                _('Delete information about recently opened files?'),
                 _('This will remove all entries from the "Recent" menu,'
                   ' and clear information about last read pages.'))
             response = dialog.run()
-            dialog.destroy()
 
             if response == gtk.RESPONSE_YES:
                 self._window.uimanager.recent.remove_all()
                 self._window.filehandler.last_read_page.clear_all()
 
+    def _create_scaling_quality_combobox(self):
+        """ Creates combo box for image scaling quality """
+        items = (
+                (_('Normal (fast)'), int(gtk.gdk.INTERP_TILES)),
+                (_('Bilinear'), int(gtk.gdk.INTERP_BILINEAR)),
+                (_('Hyperbolic (slow)'), int(gtk.gdk.INTERP_HYPER)))
+
+        selection = prefs['scaling quality']
+
+        return self._create_combobox(items, selection, self._scaling_quality_changed_cb)
+
+    def _scaling_quality_changed_cb(self, combobox, *args):
+        """ Called whan image scaling quality changes. """
+        iter = combobox.get_active_iter()
+        if combobox.get_model().iter_is_valid(iter):
+            value = combobox.get_model().get_value(iter, 1)
+            last_value = prefs['scaling quality']
+            prefs['scaling quality'] = value
+
+            if value != last_value:
+                self._window.draw_image()
+
     def _create_combobox(self, options, selected_value, change_callback):
         """ Creates a new dropdown combobox and populates it with the items
         passed in C{options}.
@@ -676,6 +767,9 @@ class _PreferencesDialog(gtk.Dialog):
         elif preference == 'number of pixels to scroll per mouse wheel event':
             prefs[preference] = int(value)
 
+        elif preference == 'smart scroll percentage':
+            prefs[preference] = value / 100.0
+
         elif preference == 'thumbnail size':
             prefs[preference] = int(value)
             self._window.thumbnailsidebar.resize()
@@ -689,6 +783,11 @@ class _PreferencesDialog(gtk.Dialog):
             prefs['number of key presses before page turn'] = int(value)
             self._window._event_handler._extra_scroll_events = 0
 
+        elif preference == 'fit to size px':
+            prefs[preference] = int(value)
+            self._window.change_zoom_mode()
+
+
     def _entry_cb(self, entry, event=None):
         """Callback for entry-type preferences."""
         text = entry.get_text()
