@@ -54,7 +54,8 @@ TRANSLATORS = (
             ('Jan Nekvasil', _('Czech translation')),
             ('Олександр Заяц', _('Ukrainian translation')),
             ('Roxerio Roxo Carrillo', _('Galician translation')),
-            ('Martin Karlsson', _('Swedish translation'))
+            ('Martin Karlsson', _('Swedish translation')),
+            ('Isratine Citizen', _('Hebrew translation'))
           )
 ARTISTS = (
             ('Victor Castillejo', _('Icon design')),
