@@ -61,6 +61,8 @@ class MainUI(gtk.UIManager):
                 'KP_Subtract', None, window.manual_zoom_out),
             ('zoom_original', gtk.STOCK_ZOOM_100, _('_Normal Size'),
                 '<Control>0', None, window.manual_zoom_original),
+            ('minimize', gtk.STOCK_LEAVE_FULLSCREEN, _('Mi_nimize'),
+                'n', None, window.minimize),
             ('close', gtk.STOCK_CLOSE, _('_Close'), None,
                 _('Closes all opened files.'), window.filehandler.close_file),
             ('quit', gtk.STOCK_QUIT, _('_Quit'),
@@ -138,6 +140,8 @@ class MainUI(gtk.UIManager):
                 'w', _('Fit width mode'), constants.ZOOM_MODE_WIDTH),
             ('fit_height_mode', 'mcomix-fitheight', _('Fit _height mode'),
                 'h', _('Fit height mode'), constants.ZOOM_MODE_HEIGHT),
+            ('fit_size_mode', 'mcomix-fitsize', _('Fit _size mode'),
+                's', _('Fit to size mode'), constants.ZOOM_MODE_SIZE),
             ('fit_manual_mode', 'mcomix-fitmanual', _('M_anual zoom mode'),
                 'a', _('Manual zoom mode'), constants.ZOOM_MODE_MANUAL)],
             3, window.change_zoom_mode)
@@ -188,6 +192,7 @@ class MainUI(gtk.UIManager):
                 <toolitem action="best_fit_mode" />
                 <toolitem action="fit_width_mode" />
                 <toolitem action="fit_height_mode" />
+                <toolitem action="fit_size_mode" />
                 <toolitem action="fit_manual_mode" />
                 <separator />
                 <toolitem action="double_page" />
@@ -208,6 +213,7 @@ class MainUI(gtk.UIManager):
                     <separator />
                     <menuitem action="delete" />
                     <separator />
+                    <menuitem action="minimize" />
                     <menuitem action="close" />
                     <menuitem action="save_and_quit" />
                     <menuitem action="quit" />
@@ -228,6 +234,7 @@ class MainUI(gtk.UIManager):
                     <menuitem action="best_fit_mode" />
                     <menuitem action="fit_width_mode" />
                     <menuitem action="fit_height_mode" />
+                    <menuitem action="fit_size_mode" />
                     <menuitem action="fit_manual_mode" />
                     <separator />
                     <menuitem action="slideshow" />
@@ -306,6 +313,7 @@ class MainUI(gtk.UIManager):
                     <menuitem action="best_fit_mode" />
                     <menuitem action="fit_width_mode" />
                     <menuitem action="fit_height_mode" />
+                    <menuitem action="fit_size_mode" />
                     <menuitem action="fit_manual_mode" />
                     <separator />
                     <menuitem action="slideshow" />
@@ -340,6 +348,7 @@ class MainUI(gtk.UIManager):
                 <separator />
                 <menuitem action="preferences" />
                 <separator />
+                <menuitem action="minimize" />
                 <menuitem action="close" />
                 <menuitem action="quit" />
             </popup>
@@ -402,6 +411,10 @@ class MainUI(gtk.UIManager):
                    'keep_transformation',
                    'enhance_image')
 
+        unavailable_file = ('next_archive',
+                              'previous_archive',
+                              'next_directory',
+                              'previous_directory')
         comment = ('comments',)
 
         general_sensitive = False
@@ -419,6 +432,10 @@ class MainUI(gtk.UIManager):
         for name in comment:
             self._actiongroup.get_action(name).set_sensitive(comment_sensitive)
 
+        if self._window.filehandler.file_load_failed:
+            for name in unavailable_file:
+                self._actiongroup.get_action(name).set_sensitive(True)
+
         self.bookmarks.set_sensitive(general_sensitive)
         self.bookmarks_popup.set_sensitive(general_sensitive)
 
